(* Exam Test Framework  *)
(* ©2009 Marwan Burelle *)

let random_printable_char =
  let ca = Char.code 'a' in
    fun _ -> Char.chr ((Random.int 26) + ca)

let rec makeList f l = function
    0 -> l
  | n when n>0 -> makeList f (f()::l) (n-1)
  | _ -> assert false

let rec insert x = function
  | (h::_) as l
      when h = x -> l
  | h::t
      when h < x -> h::insert x t
  | l
    -> x::l

let rec makeUniqList f l = function
  | 0 -> l
  | n when n>0
      -> makeUniqList f (insert (f()) l) (n-1)
  | _ -> assert false

let makePair m =
  (Random.int m, Random.int m)

let pair_to_string (x,y) =
  Printf.sprintf "(%02i,%02i)" x y

let i2s i =
  Printf.sprintf "%02d" i

let print_list_pref ?(name="l") to_s l =
  begin
    Format.printf "@[<b>%s = @[<b 2>[@;" name;
    List.iter (fun x -> Format.printf "%s;@;" (to_s x)) l;
    Format.printf "]@]@]@,";
  end

let print_list to_s l =
  begin
    Format.printf "@[<b 2>[@;";
    List.iter (fun x -> Format.printf "%s;@;" (to_s x)) l;
    Format.printf "]@]@,";
  end

class question n =
object (s)
  val q = Queue.create ()
  method getQ = q
  val mutable plist = []
  method add_plist p = plist <- p::plist
  method get_plist = plist
  method quest = "Question " ^ (string_of_int n) ^ ":\n"
  method paramline =
    (Printf.sprintf "./question%02i " n)
    ^(List.fold_left (fun s p -> s ^ p) "" plist)
  method usage = s#quest ^ s#paramline
  method beginParse =
    Format.set_margin 60;
    let anonfun s = Queue.add s q in
      Arg.parse (Arg.align []) anonfun s#usage
  method die:unit = Arg.usage (Arg.align []) (s#usage); exit 1
end

class qFile n =
object (s)
  inherit question n
  val mutable chan = None
  method getChan =
    match chan with
	None ->
	  begin
	    try
	      chan <- Some (open_in (Queue.take s#getQ));
	      s#getChan
	    with
		_ -> s#die; stdin
	  end
      | Some c -> c
  initializer
    s#add_plist "fichier "
end

class virtual qSeed =
object (s)
  method virtual getQ : string Queue.t
  method virtual usage : string
  method virtual die : unit
  method init =
    try
      Random.init (int_of_string (Queue.take s#getQ))
    with
	_ -> s#die
end

class virtual qSize =
object (s)
  method virtual getQ : string Queue.t
  method virtual usage : string
  method virtual die : unit
  val mutable size = None
  method getSize =
    match size with
	None ->
	  begin
	    try
	      size <- Some (int_of_string (Queue.take s#getQ));
	      s#getSize
	    with
		_ -> s#die; 0
	  end
      | Some n -> n
end

class qDim n =
object (s)
  inherit question n
  inherit qSize
  initializer
    s#add_plist "taille "
end

class qInt n m =
object (s)
  inherit question n
  inherit qSeed
  method getN = Random.int m
  initializer
    s#add_plist "graine "
end

class qFloat n m =
object (s)
  inherit question n
  inherit qSeed
  method getN = Random.float m
  initializer
    s#add_plist "graine "
end

class qSeedSizeInt n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method getN = Random.int m
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qSeedTwoSizeInt n m =
object (s)
  inherit question n
  inherit qSeed as qseed
  method getN = Random.int m
  val mutable size1 = -1
  val mutable size2 = -1
  method init =
    try
      qseed#init;
      size1 <- int_of_string (Queue.take s#getQ);
      size2 <- int_of_string (Queue.take s#getQ);
    with
	_ -> s#die
  method get_size1 = if size1 < 0 then s#die; size1
  method get_size2 = if size2 < 0 then s#die; size2
  initializer
    s#add_plist "taille2 ";
    s#add_plist "taille1 ";
    s#add_plist "graine "
end


class qSeedSizeMax n =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed as qseed
  val mutable maxval = 32
  method getN = Random.int maxval
  method init =
    try
      qseed#init;
      size <- Some (int_of_string (Queue.take s#getQ));
      maxval <- int_of_string (Queue.take s#getQ);
    with
	_ -> s#die
  initializer
    s#add_plist "max ";
    s#add_plist "taille ";
    s#add_plist "graine "
end

exception WrongKey

class qAsk2 n m =
object (s)
  inherit question n
  val mutable s1 = -1
  val mutable s2 = -1
  method init =
    try
      s1 <- int_of_string (Queue.take s#getQ);
      s2 <- int_of_string (Queue.take s#getQ);
      if not (s1 > 0 && s1 <= m && s2 > 0 && s2 <= m) then raise WrongKey;
    with
      | _ -> s#die
  method getS1 = s1
  method getS2 = s2
  initializer
    begin
      s#add_plist "s2 ";
      s#add_plist "s1 ";
    end
end

class qSeedSizeAsk n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed as qseed
  method getN = Random.int m
  val mutable key = None
  method getKey =
    match key with
	None ->
	  begin
	    try
	      key <- Some (int_of_string (Queue.take s#getQ));
	      s#getKey
	    with
		_ -> s#die; 0
	  end
      | Some n -> n
  method init =
    try
      qseed#init;
      size <- Some (int_of_string (Queue.take s#getQ));
      key <- Some (int_of_string (Queue.take s#getQ));
    with
	_ -> s#die
  initializer
    s#add_plist "clef ";
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qIntList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeList =
    makeList (fun () -> Random.int m) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qFloatList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeList =
    makeList (fun () -> Random.float m) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qUniqIntList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeList =
    makeUniqList (fun () -> Random.int m) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end


class qPairIntList n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  val mutable max = m
  method set_max m = max <- m
  method makeList =
    makeList (fun () -> makePair max) [] s#getSize
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qSizeString n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  method makeString =
    let s = String.create m in
      for i = 0 to m-1 do
	s.[i] <- random_printable_char ()
      done;
      s
  initializer
    s#add_plist "taille ";
    s#add_plist "graine "
end

class qSizeBoundedString n =
  let rand_print_char () =
    Char.chr (33 + Random.int 94)
  in
object (self)
  inherit question n
  inherit qSize
  inherit qSeed
  method make_string =
    begin
      let s = String.create self#getSize in
        for i = 0 to self#getSize - 1 do
	  s.[i] <- rand_print_char ()
        done;
        s
    end
  initializer
    self#add_plist "taille ";
    self#add_plist "graine "
end

class qSeedSizeOption n m =
object (s)
  inherit question n
  inherit qSize
  inherit qSeed
  val mutable opset = None
  method getN = Random.int m
  method option =
    match opset with
      | None ->
        opset <- Some (not (Queue.is_empty s#getQ));
        s#option
      | Some b -> b
  initializer
    s#add_plist " draw ";
    s#add_plist " taille ";
    s#add_plist " graine ";
end
